' ========================= TimerDown.bas ==========================

' This program runs on a PICAXE-20X2 & implements a countdown timer.
' The time (minutes & seconds) is displayed on a 4-digit, 7-segment
' LED display that is controlled by a MAX7219 LED display driver.
' A 16-key matrix keypad provides user input to set the timer.

' Hardware Interface
	symbol keypad = 7						' keypad input on ADC7 (C.3)
	symbol load   = C.2					' pulse C.0 to xfer data to 7219
	symbol piezo  = C.0					' piezo beeper on C.0

' Register addresses for the 7219
	symbol decode =  9					' decode register
	symbol brite  = 10					' LED intensity register; 15 = max
	symbol scan   = 11					' scan-limit register
	symbol on_off = 12					' 1 = display on; 0 = display off 
		
' === Variables ===
	symbol  key = w0						' ADC key value
	symbol junk = b2						' temporary variable
	symbol mins = b3						' minutes
	symbol secs = b4						' seconds
	symbol  rep = b5						' repetitions in for/next loop

	symbol   d0 = b6						' display on 1st LED from right
	symbol   d1 = b7						' display on 2nd LED from right
	symbol   d2 = b8						' display on 3rd LED from right
	symbol   d3 = b9						' display on 4th LED from right

	symbol decimal = b10				' used to insert decimal point
	symbol runflag = b11				' 0 = setting up / 1 = running

' === Directives ===
	#com 3											' specify the serial port
	#picaxe 20X2								' specify the PICAXE processor
	#no_data										' speed up the download
	#no_table										' speed up the download
	#terminal off								' disable terminal window
	
' ======================= Begin Main Program =======================

setfreq m16												' set clock speed
hspisetup spimode00,spifast				' set up hspi
dirsb = %11111111									' PortB all outputs
dirsc = %10110111									' ADC keypad input on C.3
pullup  %00000010									' enable C.6 pullup (see Manual)
adcsetup = %0000000010000000			' set up for ADC7 (see Manual)

' Initialize MAX7219
hspiout (scan,3)									' set scan limit for digits 0-3
pulsout load,1
 	 
hspiout (brite,5) 								' set brightness to 5 (15 = 100%)
pulsout load,1

hspiout (decode,15)								' set BCD decoding for digits 0-3
pulsout load,1	
 	
hspiout (on_off,1)								' turn display on
pulsout load,1

do	
	gosub UpdateDisplay

wait_for_keypress:
  	readadc keypad, junk
	if junk < 5 then wait_for_keypress

	pause 100												' debounce key
	readadc10 keypad, key						' get ADC value
		
wait_for_release:	
	readadc keypad, junk
	if junk > 5 then wait_for_release
	
	select case key									'decode key
		case < 369 : key = 10	' A
		case < 385 : key = 14	' R
		case < 402 : key = 12	' D
		case < 434 : key = 13	' L
		case < 450 : key =  0	' 0
		case < 474 : key =  9	' 9
		case < 500 : key = 15	' U
		case < 551 : key = 11	' B
		case < 587 : key =  8	' 8
		case < 634 : key =  7	' 7
		case < 674 : key =  6	' 6
		case < 733 : key =  5	' 5
		case < 804 : key =  4	' 4
		case < 885 : key =  3	' 3
		case < 977 : key =  2	' 2
		else       : key =  1	' 1
	end select
	
	select case key
	
		case 0 to 9										' digit key has been pressed
			If runflag = 0 then
				d3=d2											' rotate left and add key-press
				d2=d1
				d1=d0
				d0=key
			endif

		case 10												' "Accept" key has been pressed
			If runflag = 0 then
				runflag = 1		
				mins = 10 * d3 + d2				' initialize the timer display
				secs = 10 * d1 + d0
				settimer t1s_16 					' set timer1 to 1S interrupts
				timer = $ffff   					' interrupt on next overflow
				setintflags %10000000,%10000000  ' enable timer interrupt
			endif
			
		case 11												' "Back" key has been pressed
			If runflag = 0 then
				d0=d1											' rotate right
				d1=d2
				d2=d3
				d3=0
			else
				reset
			endif 
			
		case 12 to 15									' ignore arrow keys		

	end select
		
loop

' ============= End Main Program - Subroutines Follow ==============

Interrupt: 												' triggered by timer1 overflow
	toflag = 0   										' clear  timer overflow flag
	settimer t1s_16 								' set timer1 to 1S interrupts
	timer = $ffff  									' set interrupt for 1 major tick
	setintflags %10000000,%10000000 ' re-enable timer interrupt
			
	dec secs												' update seconds
	if secs = 255 then							' if seconds wrap from 0 to 255
		secs =  59										' then reset seconds to 59
		dec mins											' and decrement minutes
	endif
	
	d3 = mins dig 1									' 4th digit from right
	d2 = mins dig 0		`							' 3rd digit from right
	d1 = secs dig 1									' 2nd digit from right
	d0 = secs dig 0									' 1st digit from right
	
UpdateDisplay:
	hspiout (1,d3)									' 4th digit from right
	pulsout load,1

	decimal = d2 + 128	
	hspiout (2,decimal)							' 3rd digit from right
	pulsout load,1
		
	hspiout (3,d1)									' 2nd digit from right
	pulsout load,1

	hspiout (4,d0)									' 1st digit from right
	pulsout load,1	

	if runflag = 1 AND mins = 0 AND secs = 0 then gosub WaitForReset

	return


Paws:
	for junk = 1 to 120
		pause 1
		if pinC.3 = 1 then						' press "back" key to reset timer
			reset
		endif
	next junk
	return
	

WaitForReset:
	setintflags off
	adcsetup = 0										' disable ADC to use C.3 digitally
	do
		for rep = 1 to 4
			sound piezo, (115,10)
			gosub Paws
		next rep
		
		for rep = 1 to 3
			gosub Paws
		next rep
	loop
	
	return